/*-
 * Copyright 2005 Colin Percival
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#if !defined(WIN32)
#include <sys/cdefs.h>
#if defined(__FreeBSD__)
#include <sys/endian.h>
#else
#include <endian.h>
#endif
#endif
#include "sha256.h"

#if __FreeBSD_version < 500111

static __inline int be32dec( const void *pp )
{
   unsigned char const *p = ( unsigned char const * )pp;

   return ( ( p[0] << 24 ) | ( p[1] << 16 ) | ( p[2] << 8 ) | p[3] );
}

static __inline void be32enc( void *pp, int u )
{
   unsigned char *p = ( unsigned char * )pp;

   p[0] = ( u >> 24 ) & 0xff;
   p[1] = ( u >> 16 ) & 0xff;
   p[2] = ( u >> 8 ) & 0xff;
   p[3] = u & 0xff;
}

#endif

#if BYTE_ORDER == BIG_ENDIAN

/* Copy a vector of big-endian int into a vector of bytes */
#define be32enc_vect(dst, src, len)	\
	memcpy((void *)dst, (const void *)src, (size_t)len)

/* Copy a vector of bytes into a vector of big-endian int */
#define be32dec_vect(dst, src, len)	\
	memcpy((void *)dst, (const void *)src, (size_t)len)

#else /* BYTE_ORDER != BIG_ENDIAN */

/*
 * Encode a length len/4 vector of (int) into a length len vector of
 * (unsigned char) in big-endian form.  Assumes len is a multiple of 4.
 */
static void be32enc_vect( unsigned char *dst, const int *src, size_t len )
{
   size_t i;

   for( i = 0; i < len / 4; i++ )
      be32enc( dst + i * 4, src[i] );
}

/*
 * Decode a big-endian length len vector of (unsigned char) into a length
 * len/4 vector of (int).  Assumes len is a multiple of 4.
 */
static void be32dec_vect( int *dst, const unsigned char *src, size_t len )
{
   size_t i;

   for( i = 0; i < len / 4; i++ )
      dst[i] = be32dec( src + i * 4 );
}

#endif /* BYTE_ORDER != BIG_ENDIAN */

/* Elementary functions used by SHA256 */
#define Ch(x, y, z)	((x & (y ^ z)) ^ z)
#define Maj(x, y, z)	((x & (y | z)) | (y & z))
#define SHR(x, n)	(x >> n)
#define ROTR(x, n)	((x >> n) | (x << (32 - n)))
#define S0(x)		(ROTR(x, 2) ^ ROTR(x, 13) ^ ROTR(x, 22))
#define S1(x)		(ROTR(x, 6) ^ ROTR(x, 11) ^ ROTR(x, 25))
#define s0(x)		(ROTR(x, 7) ^ ROTR(x, 18) ^ SHR(x, 3))
#define s1(x)		(ROTR(x, 17) ^ ROTR(x, 19) ^ SHR(x, 10))

/* SHA256 round function */
#define RND(a, b, c, d, e, f, g, h, k)			\
	t0 = h + S1(e) + Ch(e, f, g) + k;		\
	t1 = S0(a) + Maj(a, b, c);			\
	d += t0;					\
	h  = t0 + t1;

/* Adjusted round function for rotating state */
#define RNDr(S, W, i, k)			\
	RND(S[(64 - i) % 8], S[(65 - i) % 8],	\
	    S[(66 - i) % 8], S[(67 - i) % 8],	\
	    S[(68 - i) % 8], S[(69 - i) % 8],	\
	    S[(70 - i) % 8], S[(71 - i) % 8],	\
	    W[i] + k)

/*
 * SHA256 block compression function.  The 256-bit state is transformed via
 * the 512-bit input block to produce a new state.
 */
static void SHA256_Transform( int *state, const unsigned char block[64] )
{
   int W[64];
   int S[8];
   int t0, t1;
   int i;

   /*
    * 1. Prepare message schedule W. 
    */
   be32dec_vect( W, block, 64 );
   for( i = 16; i < 64; i++ )
      W[i] = s1( W[i - 2] ) + W[i - 7] + s0( W[i - 15] ) + W[i - 16];

   /*
    * 2. Initialize working variables. 
    */
   memcpy( S, state, 32 );

   /*
    * 3. Mix. 
    */
   RNDr( S, W, 0, 0x428a2f98 );
   RNDr( S, W, 1, 0x71374491 );
   RNDr( S, W, 2, 0xb5c0fbcf );
   RNDr( S, W, 3, 0xe9b5dba5 );
   RNDr( S, W, 4, 0x3956c25b );
   RNDr( S, W, 5, 0x59f111f1 );
   RNDr( S, W, 6, 0x923f82a4 );
   RNDr( S, W, 7, 0xab1c5ed5 );
   RNDr( S, W, 8, 0xd807aa98 );
   RNDr( S, W, 9, 0x12835b01 );
   RNDr( S, W, 10, 0x243185be );
   RNDr( S, W, 11, 0x550c7dc3 );
   RNDr( S, W, 12, 0x72be5d74 );
   RNDr( S, W, 13, 0x80deb1fe );
   RNDr( S, W, 14, 0x9bdc06a7 );
   RNDr( S, W, 15, 0xc19bf174 );
   RNDr( S, W, 16, 0xe49b69c1 );
   RNDr( S, W, 17, 0xefbe4786 );
   RNDr( S, W, 18, 0x0fc19dc6 );
   RNDr( S, W, 19, 0x240ca1cc );
   RNDr( S, W, 20, 0x2de92c6f );
   RNDr( S, W, 21, 0x4a7484aa );
   RNDr( S, W, 22, 0x5cb0a9dc );
   RNDr( S, W, 23, 0x76f988da );
   RNDr( S, W, 24, 0x983e5152 );
   RNDr( S, W, 25, 0xa831c66d );
   RNDr( S, W, 26, 0xb00327c8 );
   RNDr( S, W, 27, 0xbf597fc7 );
   RNDr( S, W, 28, 0xc6e00bf3 );
   RNDr( S, W, 29, 0xd5a79147 );
   RNDr( S, W, 30, 0x06ca6351 );
   RNDr( S, W, 31, 0x14292967 );
   RNDr( S, W, 32, 0x27b70a85 );
   RNDr( S, W, 33, 0x2e1b2138 );
   RNDr( S, W, 34, 0x4d2c6dfc );
   RNDr( S, W, 35, 0x53380d13 );
   RNDr( S, W, 36, 0x650a7354 );
   RNDr( S, W, 37, 0x766a0abb );
   RNDr( S, W, 38, 0x81c2c92e );
   RNDr( S, W, 39, 0x92722c85 );
   RNDr( S, W, 40, 0xa2bfe8a1 );
   RNDr( S, W, 41, 0xa81a664b );
   RNDr( S, W, 42, 0xc24b8b70 );
   RNDr( S, W, 43, 0xc76c51a3 );
   RNDr( S, W, 44, 0xd192e819 );
   RNDr( S, W, 45, 0xd6990624 );
   RNDr( S, W, 46, 0xf40e3585 );
   RNDr( S, W, 47, 0x106aa070 );
   RNDr( S, W, 48, 0x19a4c116 );
   RNDr( S, W, 49, 0x1e376c08 );
   RNDr( S, W, 50, 0x2748774c );
   RNDr( S, W, 51, 0x34b0bcb5 );
   RNDr( S, W, 52, 0x391c0cb3 );
   RNDr( S, W, 53, 0x4ed8aa4a );
   RNDr( S, W, 54, 0x5b9cca4f );
   RNDr( S, W, 55, 0x682e6ff3 );
   RNDr( S, W, 56, 0x748f82ee );
   RNDr( S, W, 57, 0x78a5636f );
   RNDr( S, W, 58, 0x84c87814 );
   RNDr( S, W, 59, 0x8cc70208 );
   RNDr( S, W, 60, 0x90befffa );
   RNDr( S, W, 61, 0xa4506ceb );
   RNDr( S, W, 62, 0xbef9a3f7 );
   RNDr( S, W, 63, 0xc67178f2 );

   /*
    * 4. Mix local working variables into global state 
    */
   for( i = 0; i < 8; i++ )
      state[i] += S[i];
}

static unsigned char PAD[64] = {
   0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

/* Add padding and terminating bit-count. */
static void SHA256_Pad( SHA256_CTX * ctx )
{
   unsigned char len[8];
   int r, plen;

   /*
    * Convert length to a vector of bytes -- we do this now rather
    * than later because the length will change after we pad.
    */
   be32enc_vect( len, ctx->count, 8 );

   /*
    * Add 1--64 bytes so that the resulting length is 56 mod 64 
    */
   r = ( ctx->count[1] >> 3 ) & 0x3f;
   plen = ( r < 56 ) ? ( 56 - r ) : ( 120 - r );
   SHA256_Update( ctx, PAD, ( size_t ) plen );

   /*
    * Add the terminating bit-count 
    */
   SHA256_Update( ctx, len, 8 );
}

/* SHA-256 initialization.  Begins a SHA-256 operation. */
void SHA256_Init( SHA256_CTX * ctx )
{
   /*
    * Zero bits processed so far 
    */
   ctx->count[0] = ctx->count[1] = 0;

   /*
    * Magic initialization constants 
    */
   ctx->state[0] = 0x6A09E667;
   ctx->state[1] = 0xBB67AE85;
   ctx->state[2] = 0x3C6EF372;
   ctx->state[3] = 0xA54FF53A;
   ctx->state[4] = 0x510E527F;
   ctx->state[5] = 0x9B05688C;
   ctx->state[6] = 0x1F83D9AB;
   ctx->state[7] = 0x5BE0CD19;
}

/* Add bytes into the hash */
void SHA256_Update( SHA256_CTX * ctx, const unsigned char *src, size_t len )
{
   int bitlen[2];
   size_t r;

   /*
    * Number of bytes left in the buffer from previous updates 
    */
   r = ( ctx->count[1] >> 3 ) & 0x3f;

   /*
    * Convert the length into a number of bits 
    */
   bitlen[1] = ( ( int )len ) << 3;
   bitlen[0] = ( int )( len >> 29 );

   /*
    * Update number of bits 
    */
   if( ( ctx->count[1] += bitlen[1] ) < bitlen[1] )
      ctx->count[0]++;
   ctx->count[0] += bitlen[0];

   /*
    * Handle the case where we don't need to perform any transforms 
    */
   if( len < 64 - r )
   {
      memcpy( &ctx->buf[r], src, len );
      return;
   }

   /*
    * Finish the current block 
    */
   memcpy( &ctx->buf[r], src, 64 - r );
   SHA256_Transform( ctx->state, ctx->buf );
   src += 64 - r;
   len -= 64 - r;

   /*
    * Perform complete blocks 
    */
   while( len >= 64 )
   {
      SHA256_Transform( ctx->state, src );
      src += 64;
      len -= 64;
   }

   /*
    * Copy left over data into buffer 
    */
   memcpy( ctx->buf, src, len );
}

/*
 * SHA-256 finalization.  Pads the input data, exports the hash value,
 * and clears the context state.
 */
void SHA256_Final( unsigned char digest[32], SHA256_CTX * ctx )
{
   /*
    * Add padding 
    */
   SHA256_Pad( ctx );

   /*
    * Write the hash 
    */
   be32enc_vect( digest, ctx->state, 32 );

   /*
    * Clear the context state 
    */
   memset( ( void * )ctx, 0, sizeof( *ctx ) );
}

char *sha256_crypt( const char *pwd )
{
   SHA256_CTX context;
   static char output[65];
   unsigned char sha256sum[32];
   unsigned int j;

   SHA256_Init( &context );
   SHA256_Update( &context, ( const unsigned char * )pwd, strlen( pwd ) );
   SHA256_Final( sha256sum, &context );

   for( j = 0; j < 32; ++j )
   {
      snprintf( output + j * 2, 65, "%02x", sha256sum[j] );
   }
   return output;
}
